function [w,E]=fit_polynomial_map(x,t,M,alpha)
%% First estimate polynomial coefficiets w using ML estimation.
 % Then estimate precision beta of noise distribution in data t via ML.
 % Then assume 0-mean prior on w with diagonal covariance matrix alpha*I
 % where hyper-parameter alpha is a fixed input. This assumption leads to
 % the optimal regularization coefficient being lambda=alpha/beta. So new
 % optimal weights w can be computed using this optimal lambda based on
 % maximising the posterior (MAP) p(w|x,t,beta,alpha).
 % See Section 1.2.5 in Bishop's book.

N=length(x); %number of training data points

%% estimate initial w via ML estimation (no regularization)
w_ML=fit_polynomial_ML(x,t,M,0);    %corresponds to solution of Excercise 1.1
f=evaluate_polynomial(x,w_ML);
%% estimate initial beta via ML estimation (Equation 1.63 in Bishop's book)
beta=N/sum((f-t).^2);
E_ML=beta/2*sum((f-t).^2)+alpha/2*sum(w_ML.^2);
E_RMS_ML=sqrt(2*E_ML/N);

%% using MAP estimation, optimal regularizer lambda = alpha / beta
 % so now we can estimate new optimal w via the MAP estimated regulariser lambda=alpha/beta
w_MAP=-------------- ADD CODE HERE --------------;    %corresponds to solution of Excercise 1.2
f=evaluate_polynomial(x,w_MAP);
E_MAP=beta/2*sum((f-t).^2)+alpha/2*sum(w_MAP.^2);
E_RMS_MAP=sqrt(2*E_MAP/N);

%% plot results
clf;
newx=0:.01:1;
newf_ML=evaluate_polynomial(newx,w_ML);
subplot(121);
hold on;
plot(newx,sin(2*pi*newx),'g-','Linewidth',3); %original function that generated the data
plot(x,t,'o','LineWidth',2);        %original data points
plot(newx,newf_ML,'r-','Linewidth',3);    %fitted polynomial with coefficients in w*
plot(newx,newf_ML+sqrt(1/beta),'m--','Linewidth',3);    %fitted polynomial + 1 standard deviation
plot(newx,newf_ML-sqrt(1/beta),'m--','Linewidth',3);    %fitted polynomial - 1 standard deviation
axis([0 1 -2 2]);
legend Original 'Noisy data' Fitted_{ML}
title(['E_{RMS}=' num2str(E_RMS_ML) ', \beta=' num2str(beta)]);
text(.1,-1.5,['$M=$' num2str(M)],'Interpreter','Latex');
text(.1,-1.65,'$\lambda=0$','Interpreter','Latex');
xlabel x
ylabel t
drawnow;

newf_MAP=evaluate_polynomial(newx,w_MAP);
subplot(122);
hold on;
plot(newx,sin(2*pi*newx),'g-','Linewidth',3); %original function that generated the data
plot(x,t,'o','LineWidth',2);        %original data points
plot(newx,newf_MAP,'r-','Linewidth',3);    %fitted polynomial with coefficients in w*
plot(newx,newf_MAP+sqrt(1/beta),'m--','Linewidth',3);    %fitted polynomial + 1 standard deviation
plot(newx,newf_MAP-sqrt(1/beta),'m--','Linewidth',3);    %fitted polynomial - 1 standard deviation
axis([0 1 -2 2]);
legend Original 'Noisy data' Fitted_{MAP}
title(['E_{RMS}=' num2str(E_RMS_MAP) ', \beta=' num2str(beta) ', \alpha=' num2str(alpha)]);
text(.1,-1.5,['$M=$' num2str(M)],'Interpreter','Latex');
text(.1,-1.65,['$\lambda=\alpha/\beta=$' num2str(alpha/beta)],'Interpreter','Latex');
xlabel x
ylabel t
drawnow;

print(gcf,'polynomial_fitting_ML_VS_MAP','-dpng');